//
// Created by trokay on 18.06.23.
//

#include "mc0203_protocol.h"
#include "headers.h"


void encode_message(unsigned char *data, int size, int *new_size) {
    unsigned char *result = (unsigned char *) calloc(size * 2, sizeof(unsigned char));
    if (result == NULL) {
        printf("Memory allocation failed.\n");
        return;
    }
    int tmp_size = size;
    int index = 0;
    for (int i = 0; i < size; i++) {
        if (data[i] == 0x7d) {
            result[index++] = 0x7d;
            result[index++] = 0x5d;
        } else {
            if (data[i] == 0x7e) {
                result[index++] = 0x7d;
                result[index++] = 0x5e;
            } else {
                result[index++] = data[i];
            }
        }
    }

    for (int i = 0; i < index; i++) {
        data[i] = result[i];
    }
    *new_size = index;
    free(result);
}

void decode_message(unsigned char *data, int size, int *new_size) {
    unsigned char *result = (unsigned char *) calloc(size, sizeof(unsigned char));
    if (result == NULL) {
        printf("Memory allocation failed.\n");
        return;
    }
    int index = 0;
    int flag;
    for (int i = 0; i < size; i++) {
        if (data[i] == 0x7d) {
            flag = 1;
        } else {
            if (data[i] == 0x5e && flag == 1) {
                result[index++] = 0x7e;
                flag = 0;
            } else {
                if (data[i] == 0x5d && flag == 1) {
                    result[index++] = 0x7d;
                    flag = 0;
                } else {
                    result[index++] = data[i];
                }
            }
        }
    }
    for (int i = 0; i < index; ++i) {
        data[i] = result[i];
    }
    (*new_size) = index;
    free(result);
}

unsigned char calc_checksum(const char *data, int size) {
    unsigned char tmp = 0;
    while (size > 0) {
        tmp ^= data[size - 1];
        size--;
    }
    return tmp;
}

int extract_data_from_command(const char *source, int src_size, char *dest, int *dst_size) {
    unsigned char tmp_str[256];
    int tmp = 0, ind = 0;
    for (int i = 0; i < src_size && tmp < 2; i++) {
        if (source[i] == 0x7e) {
            tmp++;
        } else if (tmp == 1) {
            tmp_str[ind++] = source[i];
        }
    }
    if (tmp < 2) {
        return -1;
    }
    *dst_size = ind;

    decode_message(tmp_str, ind, &tmp);

    if (calc_checksum(tmp_str, tmp - 1) != tmp_str[tmp - 1]) {
        return -1;
    }

    for (int i = 1; i < tmp - 1; ++i) {
        dest[i - 1] = tmp_str[i];
    }
    //Command Exec
    if (tmp_str[0] < 0x80) {

    }
    tmp -= 2;

    *dst_size = tmp;
    return tmp_str[0];
}


void convert_from_string(const char *response,
                         char *dhcp,
                         char *ip,
                         char *netmask,
                         char *gw,
                         char *dns) {
    *dhcp = response[0];
    ip[0] = response[1];
    ip[1] = response[2];
    ip[2] = response[3];
    ip[3] = response[4];

    netmask[0] = response[5];
    netmask[1] = response[6];
    netmask[2] = response[7];
    netmask[3] = response[8];

    gw[0] = response[9];
    gw[1] = response[10];
    gw[2] = response[11];
    gw[3] = response[12];

    dns[0] = response[13];
    dns[1] = response[14];
    dns[2] = response[15];
    dns[3] = response[16];
}

void convert_to_string(char response[17],
                       const char *dhcp,
                       const char *ip,
                       const char *netmask,
                       const char *gw,
                       const char *dns) {
    response[0] = ip[0];
    response[1] = ip[1];
    response[2] = ip[2];
    response[3] = ip[3];

    response[4] = netmask[0];
    response[5] = netmask[1];
    response[6] = netmask[2];
    response[7] = netmask[3];

    response[8] = gw[0];
    response[9] = gw[1];
    response[10] = gw[2];
    response[11] = gw[3];

    response[12] = dns[0];
    response[13] = dns[1];
    response[14] = dns[2];
    response[15] = dns[3];

    response[16] = *dhcp;
}

/*
 * Получает данные, возвращает строку(ответ) для пересылки UART
 */
void get_response(unsigned char response[256],
                  int *response_size,
                  char *data,
                  int data_size,
                  char resp_code,
                  char index) {

    int tmp_size = data_size;
    int i = 3;
    response[0] = 0x00;
    response[1] = resp_code;
    response[2] = index;
    for (; i - 3 < data_size; ++i) {
        response[i] = data[i - 3];
    }
    unsigned char cs = calc_checksum(response, i);
    response[i++] = cs;
    encode_message(response, i, &tmp_size);
    response[0] = 0x7e;

    response[tmp_size++] = 0x7e;
    *response_size = tmp_size;
}