#!/usr/bin/python3
import sys
import struct
import serial
import array
import os

def init_serail():
    ser = serial.Serial()
    ser.port = "/dev/ttyS0"
    ser.baudrate = 115200
    ser.bytesize = serial.EIGHTBITS #number of bits per bytes
    ser.parity = serial.PARITY_NONE #set parity check: no parity
    ser.stopbits = serial.STOPBITS_ONE #number of stop bits
    ser.timeout = None          #block read
    #ser.timeout = 1            #non-block read
    #ser.timeout = 2              #timeout block read
    ser.xonxoff = False     #disable software flow control
    ser.rtscts = False     #disable hardware (RTS/CTS) flow control
    ser.dsrdtr = False       #disable hardware (DSR/DTR) flow control
    ser.writeTimeout = 2     #timeout for write
    ser.open()
    return ser

command_list = {
    0x00 : lambda x, y: get_config(x,y),
    0x01 : lambda x, y: set_config(x,y)}

cmd = {
    'cmd':'',
    'data':'',
    'csum':''}

INTERFACE_FILE = "/etc/network/interfaces"
RESOLV_FILE = "/etc/resolv.conf"

def is_start_flag(b):
    byte = struct.unpack("B",b)[0]
    if byte == 0x7e:
        return True
    else:
        return False

def read_package(ser):
    print("detect start package")
    msg = b''
    byte = ser.read()
    while not(is_start_flag(byte)):
        msg = msg + byte
        byte = ser.read()
        # line = bytes.fromhex(sys.stdin.readline().rstrip())
    msg = decode_package(msg)
    msg = array.array("B", msg)
    print(msg)
    return msg

def convert(request):
    if len(request) == 2:
        msg = {'cmnd':request[0]}
    elif len(request)>2:
        msg = {'cmnd':request[0],
               'data':request[1:-1]}
    return msg

def get_sum(data):
    sum = 0x00
    for i in data:
        sum = sum ^ i
    return sum

def check_sum(msg):
    csum = msg[-1]
    if csum == get_sum(msg[:-1]):
        return True
    else:
        return False

def encode_package(msg):
    #print(msg,"\n")
    msg = msg.replace(b'\x7d', b'\x7d\x5d')
    msg = msg.replace(b'\x7e', b'\x7d\x5e')
    #print(msg,"\n")
    return msg
    
def decode_package(msg):
    msg = msg.replace(b'\x7d\x5e', b'\x7e')
    msg = msg.replace(b'\x7d\x5d', b'\x7d')
    return msg

def applay_command(request, ser):
    command = request['cmnd']
    if 'data' in request:
        data = request['data']
    else:
        data = 0x00
    print ("[DEBUG] command ", command)
    if command in command_list:
        command_list[command](data, ser)
        
def get_config(data, ser):
    f = open(INTERFACE_FILE, "r")
    cfg = {}
    for line in f.readlines():
        x = line.split()
        if len(x)>=4:
            if x[1] == "eth0" and x[3] == "static":
                cfg['dhcp'] = [0]
            elif x[1] == "eth0" and x[3] == "dhcp":
                cfg['dhcp'] = [1]
        # elif len(x) == 2:
        #     if x[0] == "address":
        #         cfg['address'] = x[1].split(".")
        #     elif x[0] == "netmask":
        #         cfg['netmask'] = x[1].split(".")
        #     elif x[0] == "gateway":
        #         cfg['gateway'] = x[1].split(".")
    f.close()
    #if cfg['dhcp'][0] == 1:
    cfg = {}
    cmd = "ip addr show eth0|grep inet|awk '{print $2}'"
    addr = os.popen(cmd).read().strip()
    try:
        [ip, mask] = addr.split("/")
    except Exception:
        ip = "0.0.0.0"
        mask = "24"
    cfg['address'] = ip.split(".")
    m = ((0xffffffff << (32 - int(mask))) & 0xffffffff).to_bytes(4, 'big')
    cfg['netmask'] = list(struct.unpack("BBBB",m))
    cmd = "ip route|grep default|awk '{print $3}'"
    gateway = os.popen(cmd).read().strip()
    if gateway == '':
        cfg['gateway'] = [0,0,0,0]
    else:
        cfg['gateway'] = gateway.split(".")
    f = open(RESOLV_FILE, "r")
    dns = 0
    for i in f:
        d = i.split()
        if d[0] == "nameserver":
            dns = d[1]
    if dns:
        cfg['dns'] = dns.split(".")
    else:
        cfg['dns'] = [0,0,0,0]
    f.close()
    print ("get config", cfg)
    config = [0xad] + cfg['address'] + cfg['netmask'] + cfg['gateway'] + cfg['dns']
    config = list(map(int, config))
    responce = []
    for i in range(200):
        responce.append(0x00)
    for i in range(len(config)):
        responce[i] = config[i]
    # responce[199] = 0xff
    responce = [0x80] + responce
    write_response(responce, ser)
    
def set_config(data, ser):
    print(data)
    dhcp = data[0]
    address = "%s.%s.%s.%s" % (data[1], data[2], data[3], data[4])
    netmask = "%s.%s.%s.%s" % (data[5], data[6], data[7], data[8])
    gateway = "%s.%s.%s.%s" % (data[9],data[10],data[11], data[12])
    dns = "%s.%s.%s.%s" % (data[13], data[14], data[15], data[16])
    f = open(INTERFACE_FILE,"w")
    msg = "auto lo\n"
    msg += "iface lo inet loopback\n"
    msg += "auto eth0\n"
    if dhcp == 1:
        msg += "iface eth0 inet dhcp\n"
    else:
        msg += "iface eth0 inet static\n"
        msg += "\taddress " + address + "\n"
        msg += "\tnetmask " + netmask + "\n"
        msg += "\tgateway " + gateway + "\n"
        msg += "\tdns-server " + dns
    f.write(msg)    
    f.close()
    f = open(RESOLV_FILE,"w")
    msg = "nameserver " + dns
    f.write(msg)
    f.close()
    # print ("get config", msg)
    responce = [0x81]
    write_response(responce, ser)
    reload_network()
    #return responce
    
def reload_network():
    cmd = "ip -4 addr flush label 'eth0' && /etc/init.d/networking restart"
    #cmd = cmd + "; rm -rf /var/cache/fusionpbx/* && sleep 5 && systemctl restart freeswitch "
    os.system(cmd)
    
def write_response(line, ser):
    csum = get_sum(line)
    line = line + [csum]
    line = bytes(line)
    line = encode_package(line)
    line = b'\x7e' + line + b'\x7e'
    ser.write(line)
    print("responce data ", line)
    #print ("main function ", line)

    
def main():
    ser = init_serail()
    if ser.is_open:
        print("serila port open")
        while 1:
            if is_start_flag(ser.read()):
                request = read_package(ser)
                if check_sum(request):
                    request = convert(request)
                    applay_command(request, ser)
       
if __name__ == "__main__":
    main()
